@@ -1,7 +1,7 @@
 """Download pictures (or videos) along with their captions and other metadata from Instagram."""
 
 
-__version__ = '4.9.3'
+__version__ = '4.11'
 
 
 try:
@@ -18,5 +18,6 @@ from .instaloadercontext import InstaloaderContext, RateController
 from .lateststamps import LatestStamps
 from .nodeiterator import NodeIterator, FrozenNodeIterator, resumable_iteration
 from .structures import (Hashtag, Highlight, Post, PostSidecarNode, PostComment, PostCommentAnswer, PostLocation,
-                         Profile, Story, StoryItem, TopSearchResults, load_structure_from_file, save_structure_to_file,
+                         Profile, Story, StoryItem, TopSearchResults, TitlePic,
+                         load_structure_from_file, save_structure_to_file,
                          load_structure, get_json_structure)
